/**
 * Copyright (C), 2021-2021, 夸克科技有限公司
 * FileName: MvpProxyImpl
 * Author: Administrator
 * Date: 2021/10/19 11:12
 * Description:
 * History:
 * <author> <time> <version> <desc>
 * 作者姓名 修改时间 版本号 描述
 */
package quake.quake_tensorflow_face_rec.baseMvp;

import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;


import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import quake.quake_tensorflow_face_rec.baseMvp.annotation.InjectPresenter;


/**
 * @ClassName: MvpProxyImpl
 * @Description: java类作用描述
 * @Author: Administrator
 * @Date: 2021/10/19 11:12
 */
public class MvpProxyImpl<V extends IBaseView> implements IMvpProxy{
    private static final String TAG = "MvpProxyImpl";
    private V mView;
    List<BasePresenter> presenters;

    public MvpProxyImpl(V v) {
        this.mView = v;
        this.presenters = new ArrayList<>();
    }
    /**
     * 通过注解的方式生成我们的Presenter
     */
    @RequiresApi(api = Build.VERSION_CODES.N)
    @Override
    public void bindAndCreatePresenter() {
        Field[] fields = mView.getClass().getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            Log.d(TAG, "injectPresenter: field=" + fields[i]);
            boolean isAnn = fields[i].isAnnotationPresent(InjectPresenter.class);
            if (isAnn)
                Log.d(TAG, "injectPresenter: 包含指定注解");
            else Log.d(TAG, "injectPresenter: 不包含指定注解");
        }
        for (Field field : fields) {

            InjectPresenter[] injectPresenter = field.getAnnotationsByType(InjectPresenter.class);
            Log.d(TAG, "injectPresenter: 长度=" + injectPresenter.length);
            Class<? extends BasePresenter> presenterClazz = null;
            if (injectPresenter.length > 0 && injectPresenter != null) {
                Log.d(TAG, "injectPresenter: 不为空");
                try {
                    presenterClazz = (Class<? extends BasePresenter>) field.getType();

                } catch (Exception e) {
                    e.printStackTrace();
                }

                try {
                    BasePresenter basePresenter = presenterClazz.newInstance();
                    basePresenter.attach(mView);

                    field.setAccessible(true);
                    field.set(mView, basePresenter);
                    presenters.add(basePresenter);
                    basePresenter.attach(mView);
                } catch (IllegalAccessException | InstantiationException e) {
                    e.printStackTrace();
                    Log.d(TAG, "injectPresenter: 错误日志："+e.getMessage());
                }
            }
        }
    }

    @Override
    public void unbindPresenter() {
        if (presenters == null) {
            return;
        }
        for (BasePresenter basePresenter : presenters) {
            basePresenter.detach();
        }
    }
}
