package quake.quake_tensorflow_face_rec.ui.base;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.provider.Settings;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.PopupWindow;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.FragmentActivity;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.Volley;
import com.billy.android.swipe.SmartSwipe;
import com.billy.android.swipe.consumer.ActivitySlidingBackConsumer;
import com.gyf.immersionbar.ImmersionBar;

import org.json.JSONObject;


import java.util.ArrayList;
import java.util.List;

import quake.quake_tensorflow_face_rec.baseMvp.BasePresenter;
import quake.quake_tensorflow_face_rec.baseMvp.IBaseView;
import quake.quake_tensorflow_face_rec.baseMvp.IMvpProxy;
import quake.quake_tensorflow_face_rec.baseMvp.MvpProxyImpl;


public abstract class BaseActivity< P extends BasePresenter
        > extends FragmentActivity implements IBaseView {
    private Handler handler = new Handler();
    private long time = 1000 * 30;
    private PopupWindow screenPrtectPopwindow;
    private long start;
    private String dt;
    private String token;
    private boolean isWindow;

    private final String URL_CHECK_TOKEN = "http://quakelogin.quakeai.tech/oauth/check_token";
    private RequestQueue requestQueue;
    private ActivitySlidingBackConsumer mActivityConsumer;
    private AlertDialog alertDialog;
    private String TAG = "BaseActivity";


    List<BasePresenter> presenters;
    private IMvpProxy mvpProxy;


    @RequiresApi(api = Build.VERSION_CODES.N)
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        requestQueue = Volley.newRequestQueue(this);
        changeStatusBar();
        super.onCreate(savedInstanceState);
//        inflectRootView();
        //自动创建presenter并绑定

        presenters = new ArrayList<>();
        mvpProxy = new MvpProxyImpl(this);
        mvpProxy.bindAndCreatePresenter();
        //初始化控件
        initViews();
        createFunctions();

        //沉浸式状态栏
        ImmersionBar.with(this).init();
        //手指侧滑返回
        activitySlidingBack();

    }

    protected void initViews(){

    };
    protected void createFunctions(){

    };
    protected abstract P createPresenter();

    //改变状态栏
    private void changeStatusBar() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            //5.0 全透明实现
            //getWindow.setStatusBarColor(Color.TRANSPARENT)
            Window window = getWindow();
            ((Window) window).clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
            window.getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN | View.SYSTEM_UI_FLAG_LAYOUT_STABLE);
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
            window.setStatusBarColor(Color.TRANSPARENT);
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            //4.4 全透明状态栏
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mvpProxy.unbindPresenter();
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    protected void onResume() {
        super.onResume();

        checkPermission();
    }

    //侧滑返回activity
    private void activitySlidingBack() {
        SmartSwipe.wrap(this)
                .addConsumer(new ActivitySlidingBackConsumer(this))
                .setRelativeMoveFactor(0.5f)
                .enableLeft();
    }

    private void getTokenAndDt() {
        SharedPreferences sharedPreferences = getSharedPreferences("myToken", Activity.MODE_PRIVATE);
        token = sharedPreferences.getString("access_token", "");
        sharedPreferences = getSharedPreferences("configList", Activity.MODE_PRIVATE);
        dt = sharedPreferences.getString("deptId", "");
    }
    //逻辑判断

    @SuppressLint("ClickableViewAccessibility")
    public void showScreenProtectPopWindow() {

        //获得屏保界面，设置点击监听
        getTokenAndDt();
        checkToken();
    }


    private void checkToken() {
        String url = URL_CHECK_TOKEN + "?token=" + token;
        JsonObjectRequest jsonObjectRequest = new JsonObjectRequest(Request.Method.POST, url,
                null, new Response.Listener<JSONObject>() {
            @Override
            public void onResponse(JSONObject response) {

                Log.d("checkToken", "response : " + response.toString());
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Log.d("checkToken", "-----------------------------------------Error: " + error.getMessage(), error);
                if (error.networkResponse != null && error.networkResponse.data != null) {
                    VolleyError error1 = new VolleyError(new String(error.networkResponse.data));
                    error = error1;
                    Log.d("checkToken", "--------------------Error: " + error.getMessage(), error);
                }
            }
        });
        requestQueue.add(jsonObjectRequest);
    }


    private static final int REQUEST_CODE_WRITE_SETTINGS = 1;
    List<String> mPermissionList = new ArrayList<>();
    private static final int PERMISSION_REQUEST = 1;
    String[] permissions = new String[]{
            Manifest.permission.CAMERA,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.READ_EXTERNAL_STORAGE,
    };

    //检查权限并授权
    @RequiresApi(api = Build.VERSION_CODES.M)
    private void checkPermission() {
//        requestWriteSettings();
        mPermissionList.clear();

        //判断哪些权限未授予
        for (int i = 0; i < permissions.length; i++) {
            if (ContextCompat.checkSelfPermission(this, permissions[i]) != PackageManager.PERMISSION_GRANTED) {
                mPermissionList.add(permissions[i]);
            }
        }
        /**
         * 判断是否为空
         */
        if (mPermissionList.isEmpty()) {//未授予的权限为空，表示都授予了

        } else {//请求权限方法
            String[] permissions = mPermissionList.toArray(new String[mPermissionList.size()]);//将List转为数组
            for (int i = 0; i < permissions.length; i++) {
                Log.d(TAG, "checkPermission: 遍历未授权权限" + permissions[i]);
            }
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                // 选择你需要申请的权限
                for (int i = 0; i < permissions.length; i++) {
                    int state = ContextCompat.checkSelfPermission(this, permissions[i]);
                    if (state != PackageManager.PERMISSION_GRANTED) { // 判断权限的状态
                        ActivityCompat.requestPermissions(this, permissions, PERMISSION_REQUEST); // 申请权限
                        return;
                    }
                }
            }
        }


    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && requestCode == PERMISSION_REQUEST) {
            for (int i = 0; i < permissions.length; i++) {
                Log.d(TAG, "onRequestPermissionsResult: " + permissions[i] + "是否授权：" + (grantResults[i] == PackageManager.PERMISSION_GRANTED));
                if (grantResults[i] != PackageManager.PERMISSION_GRANTED) { // 用户点的拒绝，仍未拥有权限
//                    Log.d("alertDialog","alertDialog:"+(alertDialog!=null)+":"+alertDialog.isShowing());
                    if (alertDialog == null || !alertDialog.isShowing()) {
                        Toast.makeText(this, "请在设置中打开存储权限", Toast.LENGTH_SHORT).show();
                        showWarningDialog();
                    }
                    return;
                } else {
                    if (alertDialog != null) {
                        alertDialog.dismiss();
                    }
                }
            }
        }

    }

    //权限弹窗
    @RequiresApi(api = Build.VERSION_CODES.M)
    private void showWarningDialog() {
        Log.d(TAG, "showWarningDialog: 系统设置" + Settings.System.canWrite(this));
        alertDialog = new AlertDialog.Builder(this)
                .setTitle("警告！")
                .setMessage("请前往设置->应用->夸克微人脸系统->权限中打开 读写存储、定位  权限，否则功能无法正常运行！")
                .setPositiveButton("前往设置", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        // 可以选择添加如下代码在系统设置中打开该应用的设置页面
                        Intent intent = new Intent();
                        intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                        Uri uri = Uri.fromParts("package", getPackageName(), null);
                        intent.setData(uri);
                        startActivity(intent);
                    }
                }).setNegativeButton("拒绝", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        // 一般情况下如果用户不授权的话，功能是无法运行的，做退出处理
                        finish();
                    }
                }).setCancelable(false).show();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == REQUEST_CODE_WRITE_SETTINGS) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                //Settings.System.canWrite方法检测授权结果
                if (Settings.System.canWrite(getApplicationContext())) {
                    Toast.makeText(this, "获取了权限", Toast.LENGTH_SHORT).show();
                } else {
                    Toast.makeText(this, "您拒绝了权限", Toast.LENGTH_SHORT).show();
                }
            }
        }

    }

}
