package quake.quake_tensorflow_face_rec.utils;

import android.content.Context;
import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.ImageFormat;
import android.graphics.Matrix;
import android.graphics.Rect;
import android.graphics.YuvImage;
import android.graphics.drawable.BitmapDrawable;
import android.media.MediaMetadataRetriever;
import android.os.Build;
import android.util.Log;
import android.widget.ImageView;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.util.HashMap;

public class BitmapUtils {
    private static final String TAG = "BitmapUtils";

    //根据uri获取bitmap
    public static Bitmap getBitmapFromUri(String picPath){
        Bitmap bitmap = null;
        try {
            FileInputStream fis = new FileInputStream(picPath); //根据uri的path获得原图的字节流
             bitmap = BitmapFactory.decodeStream(fis);//生成原图
//                    bitmap = this.centerSquareScaleBitmap(bitmap, this.dip2px(100));//将原图裁剪
            //一定要将原图裁剪了才能显示在ImageView
            //至于为什么，我猜测是因为手机拍的照片像素至少有720*1080
            //图片一般是使用ARGB_8888，一个像素点就4个字节，直接显示原图是很费内存的，于是就无法显示了
        } catch (Exception e) {
            e.printStackTrace();
        }
        return bitmap;
    }
    //根据比例缩放图片再裁剪
    public static Bitmap cropBitmap(Bitmap srcBitmap, float w, float h,boolean center) {
        float width = srcBitmap.getWidth();//原图宽
        float height = srcBitmap.getHeight();//原图高
        Log.d(TAG, "cropBitmap: 目标宽="+w+"，高="+h);
        float x = 0, y = 0, scaleWidth = width, scaleHeight = height;
        Bitmap newBmp = null;
        if (w > h) {//如果目标宽大于目标高
            float scale = w / h;
            float tempH = width / scale;
            if (height > tempH) {
                x = 0;
                y = (height - tempH) / 2;
                scaleWidth = width;
                scaleHeight = tempH;
            } else {
                scaleWidth = height * scale;
                x = (width - scale) / 2;
                y = 0;
                scaleHeight = height;
            }
        } else if (w < h) {
            float scale = h / w;
            float tempW = height / scale;
            if (height > tempW) {
                y = 0;
                x = (height - tempW) / 2;
                scaleWidth = tempW;
                scaleHeight = height;
            } else {
                scaleHeight = width * scale;
                y = (height - scaleHeight) / 2;
                x = 0;
                scaleWidth = width;
            }
        }
        if(center){
            try {
                newBmp = Bitmap.createBitmap(srcBitmap, (int) x, (int) y, (int) scaleWidth, (int) scaleHeight, null, false);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }else {
            try {
                newBmp = Bitmap.createBitmap(srcBitmap, (int) x, 120, (int) scaleWidth, (int)scaleHeight, null, false);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return newBmp;
    }


    public static Bitmap rotateBitmap(Bitmap origin, float rotate) {
        if (origin == null) {
            return null;
        }
        int width = origin.getWidth();
        int height = origin.getHeight();
        Matrix matrix = new Matrix();
        matrix.setRotate(rotate);
        // 围绕原地进行旋转
        Bitmap newBM = Bitmap.createBitmap(origin, 0, 0, width, height, matrix, false);
        if (newBM.equals(origin)) {
            return newBM;
        }
        origin.recycle();
        return newBM;
    }

    public static byte[] rotateYUV420Degree90(byte[] data, int imageWidth, int imageHeight) {
        byte[] yuv = new byte[imageWidth * imageHeight * 3 / 2];
        // Rotate the Y luma
        int i = 0;
        for (int x = 0; x < imageWidth; x++) {
            for (int y = imageHeight - 1; y >= 0; y--) {
                yuv[i] = data[y * imageWidth + x];
                i++;
            }
        }
        // Rotate the U and V color components
        i = imageWidth * imageHeight * 3 / 2 - 1;
        for (int x = imageWidth - 1; x > 0; x = x - 2) {
            for (int y = 0; y < imageHeight / 2; y++) {
                yuv[i] = data[(imageWidth * imageHeight) + (y * imageWidth) + x];
                i--;
                yuv[i] = data[(imageWidth * imageHeight) + (y * imageWidth) + (x - 1)];
                i--;
            }
        }
        return yuv;
    }

    public static Bitmap getBitmapImageFromYUV(byte[] data, int width, int height) {
        YuvImage yuvimage = new YuvImage(data, ImageFormat.NV21, width, height, null);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        yuvimage.compressToJpeg(new Rect(0, 0, width, height), 80, baos);
        byte[] jdata = baos.toByteArray();
        BitmapFactory.Options bitmapFatoryOptions = new BitmapFactory.Options();
        bitmapFatoryOptions.inPreferredConfig = Bitmap.Config.RGB_565;
        Bitmap bmp = BitmapFactory.decodeByteArray(jdata, 0, jdata.length, bitmapFatoryOptions);
//        Bitmap bmp = BitmapFactory.decodeByteArray(jdata, 0, jdata.length);

        return bmp;
    }

    public static byte[] convertColorToByte(int color[]) {
        if (color == null) {
            return null;
        }

        byte[] data = new byte[color.length * 3];
        for (int i = 0; i < color.length; i++) {
            data[i * 3] = (byte) (color[i] >> 16 & 0xff);
            data[i * 3 + 1] = (byte) (color[i] >> 8 & 0xff);
            data[i * 3 + 2] = (byte) (color[i] & 0xff);
        }

        return data;
    }

    public static void dumpFile(String fileName, byte[] data) {
        FileOutputStream outStream;
        try {
            outStream = new FileOutputStream(fileName);
        } catch (IOException ioe) {
            throw new RuntimeException("Unable to create output file " + fileName, ioe);
        }
        try {
            outStream.write(data);
            outStream.close();
        } catch (IOException ioe) {
            throw new RuntimeException("failed writing data to file " + fileName, ioe);
        }
    }

    public static byte[] I420Tonv21(byte[] data, int width, int height) {
        byte[] ret = new byte[data.length];
        int total = width * height;

        ByteBuffer bufferY = ByteBuffer.wrap(ret, 0, total);
        ByteBuffer bufferV = ByteBuffer.wrap(ret, total, total / 4);
        ByteBuffer bufferU = ByteBuffer.wrap(ret, total + total / 4, total / 4);

        bufferY.put(data, 0, total);
        for (int i = 0; i < total / 4; i += 1) {
            bufferV.put(data[total + i]);
            bufferU.put(data[i + total + total / 4]);
        }

        return ret;
    }

    public static void saveBitmap(String path, Bitmap bitmap) {
        //获取文件
        File file = new File(path);
        FileOutputStream fos = null;

        try {
            fos = new FileOutputStream(file);
            bitmap.compress(Bitmap.CompressFormat.JPEG, 100, fos);
            fos.flush();
            Log.d("saveBitmap", "保存成功");
        } catch (IOException e) {
            e.printStackTrace();
            Log.d("saveBitmap", "保存失败");
        } finally {
            try {

                if (fos != null) {
                    fos.close();
                }

            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    //获取头像
    public static Bitmap getHttpBitmap(String url) {
        URL myFileURL;
        Bitmap bitmap = null;
        try {
            myFileURL = new URL(url);
            //获得连接
            HttpURLConnection connection = (HttpURLConnection) myFileURL.openConnection();
            //设置超时时间为6000毫秒
            connection.setConnectTimeout(6000);
            //连接设置获得数据流
            connection.setDoInput(true);
            //不使用缓存
            connection.setUseCaches(false);
            //得到数据流
            InputStream is = connection.getInputStream();
            //解析得到图片
            bitmap = BitmapFactory.decodeStream(is);
            //关闭数据流
            is.close();
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return bitmap;
    }

    public static Bitmap getBitmapFromImageView(ImageView imageView) {
        return ((BitmapDrawable) imageView.getDrawable()).getBitmap();
    }

    public static Bitmap getBitmapFromAssets(Context context, String path) {
        Bitmap bitmap = null;
        AssetManager am = context.getResources().getAssets();
        try {
            //读取assert 的文图
            InputStream is = am.open(path);
            bitmap = BitmapFactory.decodeStream(is);
        } catch (Exception e) {
        }
        return bitmap;
    }

    public static int getBitmapSize(Bitmap bitmap) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {    //API 19
            return bitmap.getAllocationByteCount();
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR1) {//API 12
            return bitmap.getByteCount();
        }
        return bitmap.getRowBytes() * bitmap.getHeight();                //earlier version
    }

    public static Bitmap getNetVideoBitmap(String videoUrl,ImageView imageView) {
        Bitmap bitmap = null;
        MediaMetadataRetriever retriever = new MediaMetadataRetriever();
        try {
            //根据url获取缩略图
            retriever.setDataSource(videoUrl, new HashMap());
            //获得第一帧图片
            bitmap = retriever.getFrameAtTime();
            if(imageView!=null){
                imageView.setImageBitmap(bitmap);
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } finally {
            retriever.release();
        }
        return bitmap;
    }
}
