/**
 * Copyright (C), 2021-2021, 夸克科技有限公司
 * FileName: FaceDetecterHelper
 * Author: Administrator
 * Date: 2021/10/28 11:57
 * Description: 人脸检测器
 * History:
 * <author> <time> <version> <desc>
 * 作者姓名 修改时间 版本号 描述
 */
package quake.quake_tensorflow_face_rec.utils.TensorFlowUtils;

import android.util.Log;

import com.google.mlkit.vision.face.FaceDetection;
import com.google.mlkit.vision.face.FaceDetector;
import com.google.mlkit.vision.face.FaceDetectorOptions;

/**
 * @ClassName: FaceDetecterHelper
 * @Description: java类作用描述
 * 人脸检测器
 * @Author: Administrator
 * @Date: 2021/10/28 11:57
 */
public class FaceDetectorHelper {
    private static final String TAG = "FaceDetectorHelper";
    private FaceDetector faceDetector;

    public FaceDetectorHelper(FaceDetector faceDetector) {
        this.faceDetector = faceDetector;
    }

    private FaceDetectorHelper() {
        this.faceDetector = createFaceDetector();
        Log.d(TAG, "FaceDetectorHelper: 人脸检测设置完成");
    }


    private volatile static FaceDetectorHelper faceDetectorHelper = null;//单例对象

    public static FaceDetectorHelper getInstance() {
        if (faceDetectorHelper == null) {
            //双重检测机制
            synchronized (FaceDetectorHelper.class) {
                //同步锁
                if (faceDetectorHelper == null) {
                    //双重检测
                    faceDetectorHelper = new FaceDetectorHelper();
                }
            }
        }
        return faceDetectorHelper;
    }

    public FaceDetector createFaceDetector() {
        FaceDetectorOptions options = setUpFaceDetector();
        //FaceDetector用于在提供的图像中查找人脸的人脸检测客户端
        return FaceDetection.getClient(options);
    }

    /**
     * Real-time contour detection of multiple faces
     * 多人脸的实时轮廓检测
     * 在人脸检测前，更改人脸检测器的默认设置使用FaceDetectorOptions
     * serPerformanceMode:定义人脸检测时，用于控制额外精度、速度权衡的扩展选项，默认快
     * setContourMode:设置是不检测轮廓还是检测定义的所有轮廓，默认无
     * setClassificationMode:指示是否运行其他分类器来表征定义的微笑和睁眼等属性，默认无
     *
     * @return
     */
    private FaceDetectorOptions setUpFaceDetector() {
        return new FaceDetectorOptions.Builder()
                .setPerformanceMode(FaceDetectorOptions.PERFORMANCE_MODE_FAST)
                .setContourMode(FaceDetectorOptions.LANDMARK_MODE_NONE)
                .setClassificationMode(FaceDetectorOptions.CLASSIFICATION_MODE_NONE)
                .build();
    }

    public FaceDetector getFaceDetector() {
        return faceDetector;
    }

    public void setFaceDetector(FaceDetector faceDetector) {
        this.faceDetector = faceDetector;
    }
}
